<?php

namespace Drupal\yasm\Utility;

use Drupal\Component\Render\FormattableMarkup;

/**
 * Yasm build html helper class.
 */
class YasmHtml {

  /**
   * Build yasm markup.
   */
  public static function markup(string $content, string $picto = NULL, array $class = []) {
    $build = [
      '#markup' => self::picto($picto) . $content,
    ];
    if (!empty($class)) {
      return [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => ['class' => $class],
        'child' => $build,
      ];
    }

    return $build;
  }

  /**
   * Build fontawesome picto.
   */
  public static function picto(string $picto = NULL) {
    if (!empty($picto)) {
      return new FormattableMarkup('<i class="@picto"></i> ', [
        '@picto' => $picto,
      ]);
    }

    return '';
  }

  /**
   * Build yasm table.
   */
  public static function table(array $header, array $rows) {
    return [
      '#type' => 'table',
      '#attributes' => [
        'class' => ['datatable', 'display'],
      ],
      '#header' => $header,
      '#rows'   => $rows,
    ];
  }

  /**
   * Build yasm titles.
   */
  public static function title(string $title, string $picto = NULL, array $class = ['title']) {
    return [
      '#markup' => new FormattableMarkup('<h4 class="@class">' . self::picto($picto) . '@title</h4>', [
        '@title' => $title,
        '@class' => implode(' ', $class),
      ]),
    ];
  }

  /**
   * Build panel container.
   */
  public static function panel($content) {
    $build = [
      '#type' => 'container',
      '#attributes' => ['class' => ['panel', 'yasm-panel']],
      'child' => [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => ['class' => ['panel__content']],
        '#value' => $content,
      ],
    ];

    if (is_string($content)) {
      $build['#value'] = $content;
    }
    else {
      $build['child'] = $content;
    }

    return $build;
  }

  /**
   * Build column container.
   */
  public static function column(array $content, $cols = 4) {
    $col_class = [
      1 => ['layout-column'],
      2 => ['layout-column', 'layout-column--half'],
      3 => ['layout-column', 'layout-column--one-third'],
      4 => ['layout-column', 'layout-column--quarter'],
    ];

    return [
      '#type' => 'container',
      '#attributes' => ['class' => $col_class[$cols]],
      'child' => $content,
    ];
  }

  /**
   * Build multiple columns with auto calculated cols.
   */
  public static function columns(array $cards, array $class = [], int $max_cols = NULL) {
    $build = [];

    // Distribuite cards i columns (default maximum cols = 4).
    $max_cols = empty($max_cols) ? 4 : $max_cols;
    $cols = count($cards);
    $cols = ($cols > $max_cols) ? $max_cols : $cols;

    $columns = [];
    $column_index = 1;
    foreach ($cards as $card) {
      if (!empty($card)) {
        // If we have more cards than columns append cards to aviable column.
        if ($column_index > $cols) {
          $column_index = 1;
        }
        $columns[$column_index][] = self::panel($card);
        $column_index++;
      }
    }

    $build = [];
    foreach ($columns as $column) {
      $build[] = self::column($column, $cols);
    }

    return [
      '#type' => 'container',
      '#attributes' => ['class' => array_merge(['yasm-columns'], $class)],
      'child' => $build,
    ];
  }

}
